from .base_classes import *
from .support_functions import *


class Questionnaire(Resource):
    """A structured set of questions and their answers. The Questionnaire
    may contain questions, answers or both. The questions are
    ordered and grouped into coherent subsets, corresponding to the
    structure of the grouping of the underlying questions.If the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, status=None, authored=None, subject=None, author=None, source=None, name=None, identifier=None, encounter=None, group=None):
        self.original_tagname_ = None
        super(Questionnaire, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.status = status
        self.authored = authored
        self.subject = subject
        self.author = author
        self.source = source
        self.name = name
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.encounter = encounter
        self.group = group
    def factory(*args_, **kwargs_):
        if Questionnaire.subclass:
            return Questionnaire.subclass(*args_, **kwargs_)
        else:
            return Questionnaire(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_authored(self): return self.authored
    def set_authored(self, authored): self.authored = authored
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_author(self): return self.author
    def set_author(self, author): self.author = author
    def get_source(self): return self.source
    def set_source(self, source): self.source = source
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_encounter(self): return self.encounter
    def set_encounter(self, encounter): self.encounter = encounter
    def get_group(self): return self.group
    def set_group(self, group): self.group = group
    def hasContent_(self):
        if (
            self.status is not None or
            self.authored is not None or
            self.subject is not None or
            self.author is not None or
            self.source is not None or
            self.name is not None or
            self.identifier or
            self.encounter is not None or
            self.group is not None or
            super(Questionnaire, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Questionnaire', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Questionnaire')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Questionnaire', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Questionnaire'):
        super(Questionnaire, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Questionnaire')
    def exportChildren(self, outfile, level, namespace_='', name_='Questionnaire', fromsubclass_=False, pretty_print=True):
        super(Questionnaire, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.authored is not None:
            self.authored.export(outfile, level, namespace_, name_='authored', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.author is not None:
            self.author.export(outfile, level, namespace_, name_='author', pretty_print=pretty_print)
        if self.source is not None:
            self.source.export(outfile, level, namespace_, name_='source', pretty_print=pretty_print)
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.encounter is not None:
            self.encounter.export(outfile, level, namespace_, name_='encounter', pretty_print=pretty_print)
        if self.group is not None:
            self.group.export(outfile, level, namespace_, name_='group', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Questionnaire', mapping_=None):
        element = super(Questionnaire, self).to_etree(parent_element, name_, mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.authored is not None:
            authored_ = self.authored
            authored_.to_etree(element, name_='authored', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.author is not None:
            author_ = self.author
            author_.to_etree(element, name_='author', mapping_=mapping_)
        if self.source is not None:
            source_ = self.source
            source_.to_etree(element, name_='source', mapping_=mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.encounter is not None:
            encounter_ = self.encounter
            encounter_.to_etree(element, name_='encounter', mapping_=mapping_)
        if self.group is not None:
            group_ = self.group
            group_.to_etree(element, name_='group', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Questionnaire, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'status':
            obj_ = QuestionnaireStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'authored':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.authored = obj_
            obj_.original_tagname_ = 'authored'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'author':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.author = obj_
            obj_.original_tagname_ = 'author'
        elif nodeName_ == 'source':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.source = obj_
            obj_.original_tagname_ = 'source'
        elif nodeName_ == 'name':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'encounter':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.encounter = obj_
            obj_.original_tagname_ = 'encounter'
        elif nodeName_ == 'group':
            obj_ = Questionnaire_Group.factory()
            obj_.build(child_)
            self.group = obj_
            obj_.original_tagname_ = 'group'
        super(Questionnaire, self).buildChildren(child_, node, nodeName_, True)
# end class Questionnaire


class Questionnaire_Group(BackboneElement):
    """A structured set of questions and their answers. The Questionnaire
    may contain questions, answers or both. The questions are
    ordered and grouped into coherent subsets, corresponding to the
    structure of the grouping of the underlying questions."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, name=None, header=None, text=None, subject=None, group=None, question=None):
        self.original_tagname_ = None
        super(Questionnaire_Group, self).__init__(id, extension, modifierExtension, )
        self.name = name
        self.header = header
        self.text = text
        self.subject = subject
        if group is None:
            self.group = []
        else:
            self.group = group
        if question is None:
            self.question = []
        else:
            self.question = question
    def factory(*args_, **kwargs_):
        if Questionnaire_Group.subclass:
            return Questionnaire_Group.subclass(*args_, **kwargs_)
        else:
            return Questionnaire_Group(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_header(self): return self.header
    def set_header(self, header): self.header = header
    def get_text(self): return self.text
    def set_text(self, text): self.text = text
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_group(self): return self.group
    def set_group(self, group): self.group = group
    def add_group(self, value): self.group.append(value)
    def insert_group(self, index, value): self.group[index] = value
    def get_question(self): return self.question
    def set_question(self, question): self.question = question
    def add_question(self, value): self.question.append(value)
    def insert_question(self, index, value): self.question[index] = value
    def hasContent_(self):
        if (
            self.name is not None or
            self.header is not None or
            self.text is not None or
            self.subject is not None or
            self.group or
            self.question or
            super(Questionnaire_Group, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Questionnaire.Group', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Questionnaire.Group')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Questionnaire.Group', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Questionnaire.Group'):
        super(Questionnaire_Group, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Questionnaire.Group')
    def exportChildren(self, outfile, level, namespace_='', name_='Questionnaire.Group', fromsubclass_=False, pretty_print=True):
        super(Questionnaire_Group, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.header is not None:
            self.header.export(outfile, level, namespace_, name_='header', pretty_print=pretty_print)
        if self.text is not None:
            self.text.export(outfile, level, namespace_, name_='text', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        for group_ in self.group:
            group_.export(outfile, level, namespace_, name_='group', pretty_print=pretty_print)
        for question_ in self.question:
            question_.export(outfile, level, namespace_, name_='question', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Questionnaire.Group', mapping_=None):
        element = super(Questionnaire_Group, self).to_etree(parent_element, name_, mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.header is not None:
            header_ = self.header
            header_.to_etree(element, name_='header', mapping_=mapping_)
        if self.text is not None:
            text_ = self.text
            text_.to_etree(element, name_='text', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        for group_ in self.group:
            group_.to_etree(element, name_='group', mapping_=mapping_)
        for question_ in self.question:
            question_.to_etree(element, name_='question', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Questionnaire_Group, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'name':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'header':
            obj_ = string.factory()
            obj_.build(child_)
            self.header = obj_
            obj_.original_tagname_ = 'header'
        elif nodeName_ == 'text':
            obj_ = string.factory()
            obj_.build(child_)
            self.text = obj_
            obj_.original_tagname_ = 'text'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'group':
            obj_ = Questionnaire_Group.factory()
            obj_.build(child_)
            self.group.append(obj_)
            obj_.original_tagname_ = 'group'
        elif nodeName_ == 'question':
            obj_ = Questionnaire_Question.factory()
            obj_.build(child_)
            self.question.append(obj_)
            obj_.original_tagname_ = 'question'
        super(Questionnaire_Group, self).buildChildren(child_, node, nodeName_, True)
# end class Questionnaire_Group


class Questionnaire_Question(BackboneElement):
    """A structured set of questions and their answers. The Questionnaire
    may contain questions, answers or both. The questions are
    ordered and grouped into coherent subsets, corresponding to the
    structure of the grouping of the underlying questions.Single-
    valued answer to the question.Structured answer in the form of a
    FHIR Resource or datatype."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, name=None, text=None, answerDecimal=None, answerInteger=None, answerBoolean=None, answerDate=None, answerString=None, answerDateTime=None, answerInstant=None, choice=None, options=None, dataBoolean=None, dataInteger=None, dataDecimal=None, dataBase64Binary=None, dataInstant=None, dataString=None, dataUri=None, dataDate=None, dataDateTime=None, dataCode=None, dataOid=None, dataUuid=None, dataId=None, dataAttachment=None, dataIdentifier=None, dataCodeableConcept=None, dataCoding=None, dataQuantity=None, dataRange=None, dataPeriod=None, dataRatio=None, dataResource=None, dataSampledData=None, dataHumanName=None, dataAddress=None, dataContact=None, dataSchedule=None, remarks=None, group=None):
        self.original_tagname_ = None
        super(Questionnaire_Question, self).__init__(id, extension, modifierExtension, )
        self.name = name
        self.text = text
        self.answerDecimal = answerDecimal
        self.answerInteger = answerInteger
        self.answerBoolean = answerBoolean
        self.answerDate = answerDate
        self.answerString = answerString
        self.answerDateTime = answerDateTime
        self.answerInstant = answerInstant
        if choice is None:
            self.choice = []
        else:
            self.choice = choice
        self.options = options
        self.dataBoolean = dataBoolean
        self.dataInteger = dataInteger
        self.dataDecimal = dataDecimal
        self.dataBase64Binary = dataBase64Binary
        self.dataInstant = dataInstant
        self.dataString = dataString
        self.dataUri = dataUri
        self.dataDate = dataDate
        self.dataDateTime = dataDateTime
        self.dataCode = dataCode
        self.dataOid = dataOid
        self.dataUuid = dataUuid
        self.dataId = dataId
        self.dataAttachment = dataAttachment
        self.dataIdentifier = dataIdentifier
        self.dataCodeableConcept = dataCodeableConcept
        self.dataCoding = dataCoding
        self.dataQuantity = dataQuantity
        self.dataRange = dataRange
        self.dataPeriod = dataPeriod
        self.dataRatio = dataRatio
        self.dataResource = dataResource
        self.dataSampledData = dataSampledData
        self.dataHumanName = dataHumanName
        self.dataAddress = dataAddress
        self.dataContact = dataContact
        self.dataSchedule = dataSchedule
        self.remarks = remarks
        if group is None:
            self.group = []
        else:
            self.group = group
    def factory(*args_, **kwargs_):
        if Questionnaire_Question.subclass:
            return Questionnaire_Question.subclass(*args_, **kwargs_)
        else:
            return Questionnaire_Question(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_text(self): return self.text
    def set_text(self, text): self.text = text
    def get_answerDecimal(self): return self.answerDecimal
    def set_answerDecimal(self, answerDecimal): self.answerDecimal = answerDecimal
    def get_answerInteger(self): return self.answerInteger
    def set_answerInteger(self, answerInteger): self.answerInteger = answerInteger
    def get_answerBoolean(self): return self.answerBoolean
    def set_answerBoolean(self, answerBoolean): self.answerBoolean = answerBoolean
    def get_answerDate(self): return self.answerDate
    def set_answerDate(self, answerDate): self.answerDate = answerDate
    def get_answerString(self): return self.answerString
    def set_answerString(self, answerString): self.answerString = answerString
    def get_answerDateTime(self): return self.answerDateTime
    def set_answerDateTime(self, answerDateTime): self.answerDateTime = answerDateTime
    def get_answerInstant(self): return self.answerInstant
    def set_answerInstant(self, answerInstant): self.answerInstant = answerInstant
    def get_choice(self): return self.choice
    def set_choice(self, choice): self.choice = choice
    def add_choice(self, value): self.choice.append(value)
    def insert_choice(self, index, value): self.choice[index] = value
    def get_options(self): return self.options
    def set_options(self, options): self.options = options
    def get_dataBoolean(self): return self.dataBoolean
    def set_dataBoolean(self, dataBoolean): self.dataBoolean = dataBoolean
    def get_dataInteger(self): return self.dataInteger
    def set_dataInteger(self, dataInteger): self.dataInteger = dataInteger
    def get_dataDecimal(self): return self.dataDecimal
    def set_dataDecimal(self, dataDecimal): self.dataDecimal = dataDecimal
    def get_dataBase64Binary(self): return self.dataBase64Binary
    def set_dataBase64Binary(self, dataBase64Binary): self.dataBase64Binary = dataBase64Binary
    def get_dataInstant(self): return self.dataInstant
    def set_dataInstant(self, dataInstant): self.dataInstant = dataInstant
    def get_dataString(self): return self.dataString
    def set_dataString(self, dataString): self.dataString = dataString
    def get_dataUri(self): return self.dataUri
    def set_dataUri(self, dataUri): self.dataUri = dataUri
    def get_dataDate(self): return self.dataDate
    def set_dataDate(self, dataDate): self.dataDate = dataDate
    def get_dataDateTime(self): return self.dataDateTime
    def set_dataDateTime(self, dataDateTime): self.dataDateTime = dataDateTime
    def get_dataCode(self): return self.dataCode
    def set_dataCode(self, dataCode): self.dataCode = dataCode
    def get_dataOid(self): return self.dataOid
    def set_dataOid(self, dataOid): self.dataOid = dataOid
    def get_dataUuid(self): return self.dataUuid
    def set_dataUuid(self, dataUuid): self.dataUuid = dataUuid
    def get_dataId(self): return self.dataId
    def set_dataId(self, dataId): self.dataId = dataId
    def get_dataAttachment(self): return self.dataAttachment
    def set_dataAttachment(self, dataAttachment): self.dataAttachment = dataAttachment
    def get_dataIdentifier(self): return self.dataIdentifier
    def set_dataIdentifier(self, dataIdentifier): self.dataIdentifier = dataIdentifier
    def get_dataCodeableConcept(self): return self.dataCodeableConcept
    def set_dataCodeableConcept(self, dataCodeableConcept): self.dataCodeableConcept = dataCodeableConcept
    def get_dataCoding(self): return self.dataCoding
    def set_dataCoding(self, dataCoding): self.dataCoding = dataCoding
    def get_dataQuantity(self): return self.dataQuantity
    def set_dataQuantity(self, dataQuantity): self.dataQuantity = dataQuantity
    def get_dataRange(self): return self.dataRange
    def set_dataRange(self, dataRange): self.dataRange = dataRange
    def get_dataPeriod(self): return self.dataPeriod
    def set_dataPeriod(self, dataPeriod): self.dataPeriod = dataPeriod
    def get_dataRatio(self): return self.dataRatio
    def set_dataRatio(self, dataRatio): self.dataRatio = dataRatio
    def get_dataResource(self): return self.dataResource
    def set_dataResource(self, dataResource): self.dataResource = dataResource
    def get_dataSampledData(self): return self.dataSampledData
    def set_dataSampledData(self, dataSampledData): self.dataSampledData = dataSampledData
    def get_dataHumanName(self): return self.dataHumanName
    def set_dataHumanName(self, dataHumanName): self.dataHumanName = dataHumanName
    def get_dataAddress(self): return self.dataAddress
    def set_dataAddress(self, dataAddress): self.dataAddress = dataAddress
    def get_dataContact(self): return self.dataContact
    def set_dataContact(self, dataContact): self.dataContact = dataContact
    def get_dataSchedule(self): return self.dataSchedule
    def set_dataSchedule(self, dataSchedule): self.dataSchedule = dataSchedule
    def get_remarks(self): return self.remarks
    def set_remarks(self, remarks): self.remarks = remarks
    def get_group(self): return self.group
    def set_group(self, group): self.group = group
    def add_group(self, value): self.group.append(value)
    def insert_group(self, index, value): self.group[index] = value
    def hasContent_(self):
        if (
            self.name is not None or
            self.text is not None or
            self.answerDecimal is not None or
            self.answerInteger is not None or
            self.answerBoolean is not None or
            self.answerDate is not None or
            self.answerString is not None or
            self.answerDateTime is not None or
            self.answerInstant is not None or
            self.choice or
            self.options is not None or
            self.dataBoolean is not None or
            self.dataInteger is not None or
            self.dataDecimal is not None or
            self.dataBase64Binary is not None or
            self.dataInstant is not None or
            self.dataString is not None or
            self.dataUri is not None or
            self.dataDate is not None or
            self.dataDateTime is not None or
            self.dataCode is not None or
            self.dataOid is not None or
            self.dataUuid is not None or
            self.dataId is not None or
            self.dataAttachment is not None or
            self.dataIdentifier is not None or
            self.dataCodeableConcept is not None or
            self.dataCoding is not None or
            self.dataQuantity is not None or
            self.dataRange is not None or
            self.dataPeriod is not None or
            self.dataRatio is not None or
            self.dataResource is not None or
            self.dataSampledData is not None or
            self.dataHumanName is not None or
            self.dataAddress is not None or
            self.dataContact is not None or
            self.dataSchedule is not None or
            self.remarks is not None or
            self.group or
            super(Questionnaire_Question, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Questionnaire.Question', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Questionnaire.Question')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Questionnaire.Question', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Questionnaire.Question'):
        super(Questionnaire_Question, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Questionnaire.Question')
    def exportChildren(self, outfile, level, namespace_='', name_='Questionnaire.Question', fromsubclass_=False, pretty_print=True):
        super(Questionnaire_Question, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.text is not None:
            self.text.export(outfile, level, namespace_, name_='text', pretty_print=pretty_print)
        if self.answerDecimal is not None:
            self.answerDecimal.export(outfile, level, namespace_, name_='answerDecimal', pretty_print=pretty_print)
        if self.answerInteger is not None:
            self.answerInteger.export(outfile, level, namespace_, name_='answerInteger', pretty_print=pretty_print)
        if self.answerBoolean is not None:
            self.answerBoolean.export(outfile, level, namespace_, name_='answerBoolean', pretty_print=pretty_print)
        if self.answerDate is not None:
            self.answerDate.export(outfile, level, namespace_, name_='answerDate', pretty_print=pretty_print)
        if self.answerString is not None:
            self.answerString.export(outfile, level, namespace_, name_='answerString', pretty_print=pretty_print)
        if self.answerDateTime is not None:
            self.answerDateTime.export(outfile, level, namespace_, name_='answerDateTime', pretty_print=pretty_print)
        if self.answerInstant is not None:
            self.answerInstant.export(outfile, level, namespace_, name_='answerInstant', pretty_print=pretty_print)
        for choice_ in self.choice:
            choice_.export(outfile, level, namespace_, name_='choice', pretty_print=pretty_print)
        if self.options is not None:
            self.options.export(outfile, level, namespace_, name_='options', pretty_print=pretty_print)
        if self.dataBoolean is not None:
            self.dataBoolean.export(outfile, level, namespace_, name_='dataBoolean', pretty_print=pretty_print)
        if self.dataInteger is not None:
            self.dataInteger.export(outfile, level, namespace_, name_='dataInteger', pretty_print=pretty_print)
        if self.dataDecimal is not None:
            self.dataDecimal.export(outfile, level, namespace_, name_='dataDecimal', pretty_print=pretty_print)
        if self.dataBase64Binary is not None:
            self.dataBase64Binary.export(outfile, level, namespace_, name_='dataBase64Binary', pretty_print=pretty_print)
        if self.dataInstant is not None:
            self.dataInstant.export(outfile, level, namespace_, name_='dataInstant', pretty_print=pretty_print)
        if self.dataString is not None:
            self.dataString.export(outfile, level, namespace_, name_='dataString', pretty_print=pretty_print)
        if self.dataUri is not None:
            self.dataUri.export(outfile, level, namespace_, name_='dataUri', pretty_print=pretty_print)
        if self.dataDate is not None:
            self.dataDate.export(outfile, level, namespace_, name_='dataDate', pretty_print=pretty_print)
        if self.dataDateTime is not None:
            self.dataDateTime.export(outfile, level, namespace_, name_='dataDateTime', pretty_print=pretty_print)
        if self.dataCode is not None:
            self.dataCode.export(outfile, level, namespace_, name_='dataCode', pretty_print=pretty_print)
        if self.dataOid is not None:
            self.dataOid.export(outfile, level, namespace_, name_='dataOid', pretty_print=pretty_print)
        if self.dataUuid is not None:
            self.dataUuid.export(outfile, level, namespace_, name_='dataUuid', pretty_print=pretty_print)
        if self.dataId is not None:
            self.dataId.export(outfile, level, namespace_, name_='dataId', pretty_print=pretty_print)
        if self.dataAttachment is not None:
            self.dataAttachment.export(outfile, level, namespace_, name_='dataAttachment', pretty_print=pretty_print)
        if self.dataIdentifier is not None:
            self.dataIdentifier.export(outfile, level, namespace_, name_='dataIdentifier', pretty_print=pretty_print)
        if self.dataCodeableConcept is not None:
            self.dataCodeableConcept.export(outfile, level, namespace_, name_='dataCodeableConcept', pretty_print=pretty_print)
        if self.dataCoding is not None:
            self.dataCoding.export(outfile, level, namespace_, name_='dataCoding', pretty_print=pretty_print)
        if self.dataQuantity is not None:
            self.dataQuantity.export(outfile, level, namespace_, name_='dataQuantity', pretty_print=pretty_print)
        if self.dataRange is not None:
            self.dataRange.export(outfile, level, namespace_, name_='dataRange', pretty_print=pretty_print)
        if self.dataPeriod is not None:
            self.dataPeriod.export(outfile, level, namespace_, name_='dataPeriod', pretty_print=pretty_print)
        if self.dataRatio is not None:
            self.dataRatio.export(outfile, level, namespace_, name_='dataRatio', pretty_print=pretty_print)
        if self.dataResource is not None:
            self.dataResource.export(outfile, level, namespace_, name_='dataResource', pretty_print=pretty_print)
        if self.dataSampledData is not None:
            self.dataSampledData.export(outfile, level, namespace_, name_='dataSampledData', pretty_print=pretty_print)
        if self.dataHumanName is not None:
            self.dataHumanName.export(outfile, level, namespace_, name_='dataHumanName', pretty_print=pretty_print)
        if self.dataAddress is not None:
            self.dataAddress.export(outfile, level, namespace_, name_='dataAddress', pretty_print=pretty_print)
        if self.dataContact is not None:
            self.dataContact.export(outfile, level, namespace_, name_='dataContact', pretty_print=pretty_print)
        if self.dataSchedule is not None:
            self.dataSchedule.export(outfile, level, namespace_, name_='dataSchedule', pretty_print=pretty_print)
        if self.remarks is not None:
            self.remarks.export(outfile, level, namespace_, name_='remarks', pretty_print=pretty_print)
        for group_ in self.group:
            group_.export(outfile, level, namespace_, name_='group', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Questionnaire.Question', mapping_=None):
        element = super(Questionnaire_Question, self).to_etree(parent_element, name_, mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.text is not None:
            text_ = self.text
            text_.to_etree(element, name_='text', mapping_=mapping_)
        if self.answerDecimal is not None:
            answerDecimal_ = self.answerDecimal
            answerDecimal_.to_etree(element, name_='answerDecimal', mapping_=mapping_)
        if self.answerInteger is not None:
            answerInteger_ = self.answerInteger
            answerInteger_.to_etree(element, name_='answerInteger', mapping_=mapping_)
        if self.answerBoolean is not None:
            answerBoolean_ = self.answerBoolean
            answerBoolean_.to_etree(element, name_='answerBoolean', mapping_=mapping_)
        if self.answerDate is not None:
            answerDate_ = self.answerDate
            answerDate_.to_etree(element, name_='answerDate', mapping_=mapping_)
        if self.answerString is not None:
            answerString_ = self.answerString
            answerString_.to_etree(element, name_='answerString', mapping_=mapping_)
        if self.answerDateTime is not None:
            answerDateTime_ = self.answerDateTime
            answerDateTime_.to_etree(element, name_='answerDateTime', mapping_=mapping_)
        if self.answerInstant is not None:
            answerInstant_ = self.answerInstant
            answerInstant_.to_etree(element, name_='answerInstant', mapping_=mapping_)
        for choice_ in self.choice:
            choice_.to_etree(element, name_='choice', mapping_=mapping_)
        if self.options is not None:
            options_ = self.options
            options_.to_etree(element, name_='options', mapping_=mapping_)
        if self.dataBoolean is not None:
            dataBoolean_ = self.dataBoolean
            dataBoolean_.to_etree(element, name_='dataBoolean', mapping_=mapping_)
        if self.dataInteger is not None:
            dataInteger_ = self.dataInteger
            dataInteger_.to_etree(element, name_='dataInteger', mapping_=mapping_)
        if self.dataDecimal is not None:
            dataDecimal_ = self.dataDecimal
            dataDecimal_.to_etree(element, name_='dataDecimal', mapping_=mapping_)
        if self.dataBase64Binary is not None:
            dataBase64Binary_ = self.dataBase64Binary
            dataBase64Binary_.to_etree(element, name_='dataBase64Binary', mapping_=mapping_)
        if self.dataInstant is not None:
            dataInstant_ = self.dataInstant
            dataInstant_.to_etree(element, name_='dataInstant', mapping_=mapping_)
        if self.dataString is not None:
            dataString_ = self.dataString
            dataString_.to_etree(element, name_='dataString', mapping_=mapping_)
        if self.dataUri is not None:
            dataUri_ = self.dataUri
            dataUri_.to_etree(element, name_='dataUri', mapping_=mapping_)
        if self.dataDate is not None:
            dataDate_ = self.dataDate
            dataDate_.to_etree(element, name_='dataDate', mapping_=mapping_)
        if self.dataDateTime is not None:
            dataDateTime_ = self.dataDateTime
            dataDateTime_.to_etree(element, name_='dataDateTime', mapping_=mapping_)
        if self.dataCode is not None:
            dataCode_ = self.dataCode
            dataCode_.to_etree(element, name_='dataCode', mapping_=mapping_)
        if self.dataOid is not None:
            dataOid_ = self.dataOid
            dataOid_.to_etree(element, name_='dataOid', mapping_=mapping_)
        if self.dataUuid is not None:
            dataUuid_ = self.dataUuid
            dataUuid_.to_etree(element, name_='dataUuid', mapping_=mapping_)
        if self.dataId is not None:
            dataId_ = self.dataId
            dataId_.to_etree(element, name_='dataId', mapping_=mapping_)
        if self.dataAttachment is not None:
            dataAttachment_ = self.dataAttachment
            dataAttachment_.to_etree(element, name_='dataAttachment', mapping_=mapping_)
        if self.dataIdentifier is not None:
            dataIdentifier_ = self.dataIdentifier
            dataIdentifier_.to_etree(element, name_='dataIdentifier', mapping_=mapping_)
        if self.dataCodeableConcept is not None:
            dataCodeableConcept_ = self.dataCodeableConcept
            dataCodeableConcept_.to_etree(element, name_='dataCodeableConcept', mapping_=mapping_)
        if self.dataCoding is not None:
            dataCoding_ = self.dataCoding
            dataCoding_.to_etree(element, name_='dataCoding', mapping_=mapping_)
        if self.dataQuantity is not None:
            dataQuantity_ = self.dataQuantity
            dataQuantity_.to_etree(element, name_='dataQuantity', mapping_=mapping_)
        if self.dataRange is not None:
            dataRange_ = self.dataRange
            dataRange_.to_etree(element, name_='dataRange', mapping_=mapping_)
        if self.dataPeriod is not None:
            dataPeriod_ = self.dataPeriod
            dataPeriod_.to_etree(element, name_='dataPeriod', mapping_=mapping_)
        if self.dataRatio is not None:
            dataRatio_ = self.dataRatio
            dataRatio_.to_etree(element, name_='dataRatio', mapping_=mapping_)
        if self.dataResource is not None:
            dataResource_ = self.dataResource
            dataResource_.to_etree(element, name_='dataResource', mapping_=mapping_)
        if self.dataSampledData is not None:
            dataSampledData_ = self.dataSampledData
            dataSampledData_.to_etree(element, name_='dataSampledData', mapping_=mapping_)
        if self.dataHumanName is not None:
            dataHumanName_ = self.dataHumanName
            dataHumanName_.to_etree(element, name_='dataHumanName', mapping_=mapping_)
        if self.dataAddress is not None:
            dataAddress_ = self.dataAddress
            dataAddress_.to_etree(element, name_='dataAddress', mapping_=mapping_)
        if self.dataContact is not None:
            dataContact_ = self.dataContact
            dataContact_.to_etree(element, name_='dataContact', mapping_=mapping_)
        if self.dataSchedule is not None:
            dataSchedule_ = self.dataSchedule
            dataSchedule_.to_etree(element, name_='dataSchedule', mapping_=mapping_)
        if self.remarks is not None:
            remarks_ = self.remarks
            remarks_.to_etree(element, name_='remarks', mapping_=mapping_)
        for group_ in self.group:
            group_.to_etree(element, name_='group', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Questionnaire_Question, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'name':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'text':
            obj_ = string.factory()
            obj_.build(child_)
            self.text = obj_
            obj_.original_tagname_ = 'text'
        elif nodeName_ == 'answerDecimal':
            obj_ = decimal.factory()
            obj_.build(child_)
            self.answerDecimal = obj_
            obj_.original_tagname_ = 'answerDecimal'
        elif nodeName_ == 'answerInteger':
            obj_ = integer.factory()
            obj_.build(child_)
            self.answerInteger = obj_
            obj_.original_tagname_ = 'answerInteger'
        elif nodeName_ == 'answerBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.answerBoolean = obj_
            obj_.original_tagname_ = 'answerBoolean'
        elif nodeName_ == 'answerDate':
            obj_ = date.factory()
            obj_.build(child_)
            self.answerDate = obj_
            obj_.original_tagname_ = 'answerDate'
        elif nodeName_ == 'answerString':
            obj_ = string.factory()
            obj_.build(child_)
            self.answerString = obj_
            obj_.original_tagname_ = 'answerString'
        elif nodeName_ == 'answerDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.answerDateTime = obj_
            obj_.original_tagname_ = 'answerDateTime'
        elif nodeName_ == 'answerInstant':
            obj_ = instant.factory()
            obj_.build(child_)
            self.answerInstant = obj_
            obj_.original_tagname_ = 'answerInstant'
        elif nodeName_ == 'choice':
            obj_ = Coding.factory()
            obj_.build(child_)
            self.choice.append(obj_)
            obj_.original_tagname_ = 'choice'
        elif nodeName_ == 'options':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.options = obj_
            obj_.original_tagname_ = 'options'
        elif nodeName_ == 'dataBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.dataBoolean = obj_
            obj_.original_tagname_ = 'dataBoolean'
        elif nodeName_ == 'dataInteger':
            obj_ = integer.factory()
            obj_.build(child_)
            self.dataInteger = obj_
            obj_.original_tagname_ = 'dataInteger'
        elif nodeName_ == 'dataDecimal':
            obj_ = decimal.factory()
            obj_.build(child_)
            self.dataDecimal = obj_
            obj_.original_tagname_ = 'dataDecimal'
        elif nodeName_ == 'dataBase64Binary':
            obj_ = base64Binary.factory()
            obj_.build(child_)
            self.dataBase64Binary = obj_
            obj_.original_tagname_ = 'dataBase64Binary'
        elif nodeName_ == 'dataInstant':
            obj_ = instant.factory()
            obj_.build(child_)
            self.dataInstant = obj_
            obj_.original_tagname_ = 'dataInstant'
        elif nodeName_ == 'dataString':
            obj_ = string.factory()
            obj_.build(child_)
            self.dataString = obj_
            obj_.original_tagname_ = 'dataString'
        elif nodeName_ == 'dataUri':
            obj_ = uri.factory()
            obj_.build(child_)
            self.dataUri = obj_
            obj_.original_tagname_ = 'dataUri'
        elif nodeName_ == 'dataDate':
            obj_ = date.factory()
            obj_.build(child_)
            self.dataDate = obj_
            obj_.original_tagname_ = 'dataDate'
        elif nodeName_ == 'dataDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.dataDateTime = obj_
            obj_.original_tagname_ = 'dataDateTime'
        elif nodeName_ == 'dataCode':
            obj_ = code.factory()
            obj_.build(child_)
            self.dataCode = obj_
            obj_.original_tagname_ = 'dataCode'
        elif nodeName_ == 'dataOid':
            obj_ = oid.factory()
            obj_.build(child_)
            self.dataOid = obj_
            obj_.original_tagname_ = 'dataOid'
        elif nodeName_ == 'dataUuid':
            obj_ = uuid.factory()
            obj_.build(child_)
            self.dataUuid = obj_
            obj_.original_tagname_ = 'dataUuid'
        elif nodeName_ == 'dataId':
            obj_ = id.factory()
            obj_.build(child_)
            self.dataId = obj_
            obj_.original_tagname_ = 'dataId'
        elif nodeName_ == 'dataAttachment':
            obj_ = Attachment.factory()
            obj_.build(child_)
            self.dataAttachment = obj_
            obj_.original_tagname_ = 'dataAttachment'
        elif nodeName_ == 'dataIdentifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.dataIdentifier = obj_
            obj_.original_tagname_ = 'dataIdentifier'
        elif nodeName_ == 'dataCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.dataCodeableConcept = obj_
            obj_.original_tagname_ = 'dataCodeableConcept'
        elif nodeName_ == 'dataCoding':
            obj_ = Coding.factory()
            obj_.build(child_)
            self.dataCoding = obj_
            obj_.original_tagname_ = 'dataCoding'
        elif nodeName_ == 'dataQuantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.dataQuantity = obj_
            obj_.original_tagname_ = 'dataQuantity'
        elif nodeName_ == 'dataRange':
            obj_ = Range.factory()
            obj_.build(child_)
            self.dataRange = obj_
            obj_.original_tagname_ = 'dataRange'
        elif nodeName_ == 'dataPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.dataPeriod = obj_
            obj_.original_tagname_ = 'dataPeriod'
        elif nodeName_ == 'dataRatio':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.dataRatio = obj_
            obj_.original_tagname_ = 'dataRatio'
        elif nodeName_ == 'dataResource':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.dataResource = obj_
            obj_.original_tagname_ = 'dataResource'
        elif nodeName_ == 'dataSampledData':
            obj_ = SampledData.factory()
            obj_.build(child_)
            self.dataSampledData = obj_
            obj_.original_tagname_ = 'dataSampledData'
        elif nodeName_ == 'dataHumanName':
            obj_ = HumanName.factory()
            obj_.build(child_)
            self.dataHumanName = obj_
            obj_.original_tagname_ = 'dataHumanName'
        elif nodeName_ == 'dataAddress':
            obj_ = Address.factory()
            obj_.build(child_)
            self.dataAddress = obj_
            obj_.original_tagname_ = 'dataAddress'
        elif nodeName_ == 'dataContact':
            obj_ = Contact.factory()
            obj_.build(child_)
            self.dataContact = obj_
            obj_.original_tagname_ = 'dataContact'
        elif nodeName_ == 'dataSchedule':
            obj_ = Schedule.factory()
            obj_.build(child_)
            self.dataSchedule = obj_
            obj_.original_tagname_ = 'dataSchedule'
        elif nodeName_ == 'remarks':
            obj_ = string.factory()
            obj_.build(child_)
            self.remarks = obj_
            obj_.original_tagname_ = 'remarks'
        elif nodeName_ == 'group':
            obj_ = Questionnaire_Group.factory()
            obj_.build(child_)
            self.group.append(obj_)
            obj_.original_tagname_ = 'group'
        super(Questionnaire_Question, self).buildChildren(child_, node, nodeName_, True)
# end class Questionnaire_Question


class QuestionnaireStatus(Element):
    """Lifecycle status of the questionnaireIf the element is present, it
    must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(QuestionnaireStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if QuestionnaireStatus.subclass:
            return QuestionnaireStatus.subclass(*args_, **kwargs_)
        else:
            return QuestionnaireStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(QuestionnaireStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='QuestionnaireStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='QuestionnaireStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='QuestionnaireStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='QuestionnaireStatus'):
        super(QuestionnaireStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='QuestionnaireStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='QuestionnaireStatus', fromsubclass_=False, pretty_print=True):
        super(QuestionnaireStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='QuestionnaireStatus', mapping_=None):
        element = super(QuestionnaireStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(QuestionnaireStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(QuestionnaireStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class QuestionnaireStatus



from StringIO import StringIO
from .datastore import find_record
from operator import attrgetter
import server.fhir as supermod
from server.common import safe_attrgetter
from .health_mixin import ExportXMLMixin

try:
    from flask import url_for
    RUN_FLASK=True
except:
    from .datastore import dumb_url_generate
    RUN_FLASK=False

class FieldError(Exception): pass

class Observation_Map:
    """This class holds the mapping between GNU Health and FHIR
        for the Observation resource
    """
    model_mapping = {
        'gnuhealth.lab.test.critearea':
            {'patient': 'gnuhealth_lab_id.patient',
            'date': 'gnuhealth_lab_id.date_analysis',
            'performer': 'pathologist',
            'comments': 'remarks',
            'value': 'result',
            'excluded': 'excluded'},
        'gnuhealth.icu.apache2':
            { 'patient': 'name.patient',
            'date': 'score_date',
            'fields': {
                'rate': 'respiratory_rate',
                'pulse': 'heart_rate',
                'temp': 'temperature'}},
        'gnuhealth.patient.ambulatory_care':
            { 'patient': 'patient',
            'comments': 'session_notes',
            'date': 'session_start',
            'performer': 'health_professional',
            'fields': {
                'rate': 'respiratory_rate',
                'pulse': 'bpm',
                'temp': 'temperature',
                'press_d': 'diastolic',
                'press_s': 'systolic'}},
        'gnuhealth.patient.evaluation':
            {'patient': 'patient',
            'comments': 'notes',
            'date': 'evaluation_start',
            'performer': 'healthprof',
            'fields': {
                'rate': 'respiratory_rate',
                'pulse': 'bpm',
                'temp': 'temperature',
                'press_d': 'diastolic',
                'press_s': 'systolic'}},
        'gnuhealth.patient.rounding':
            {'patient': 'name.patient',
            'comments': 'round_summary',
            'date': 'evaluation_start',
            'performer': 'health_professional',
            'fields': {
                'rate': 'respiratory_rate',
                'pulse': 'bpm',
                'temp': 'temperature',
                'press_d': 'diastolic',
                'press_s': 'systolic'}}}

    url_prefixes ={}
            #'gnuhealth.lab.test.critearea': 'lab',
            #'gnuhealth.patient.rounding': 'rounds',
            #'gnuhealth.patient.evaluation': 'eval',
            #'gnuhealth.patient.ambulatory_care': 'amb',
            #'gnuhealth.icu.apache2': 'icu'}

    resource_search_params = {
                    '_id': 'token',
                    '_language': None,
                    'date': 'date',
                    'name': 'token',
                    'performer': None,
                    'reliability': None,
                    'related': None,
                    'related-target': None,
                    'related-type': None,
                    'specimen': None,
                    'status': None,
                    'subject': 'reference',
                    'value-concept': None,
                    'value-date': None,
                    'value-quantity': 'quantity',
                    'value-string': None}

    # Must be attached to lab
    root_search=[[('gnuhealth_lab_id', '!=', None)]]
    #root_search=[['OR', [('result', '!=', None)],
                        #[('excluded', '=', True)],
                        #[('gnuhealth_lab_id', '!=', None)]]]

    # Maps reference parameters to correct resource
    chain_map={
                'subject': 'Patient'}

    search_mapping ={
                    '_id': ['id'],
                    'date': ['gnuhealth_lab_id.date_analysis'],
                    'name': ['name'],
                    'subject': ['gnuhealth_lab_id.patient'],
                    'value-quantity': ['result']}

    todo_search_mapping={
           'gnuhealth.patient.evaluation': {'_id': (['id'], 'token'), #Needs to be parsed MODEL-ID-FIELD
                    '_language': None,
                    'date': None,
                    # These values point to keys on the 'fields' dict
                    'name': ({'Respiratory rate': 'rate',
                                'Heart rate': 'pulse', 'Systolic pressure': 'press_s',
                                'Diastolic pressure': 'press_d',
                                'Temperature': 'temp'}, 'user-defined'),
                    'performer': (['healthprof'], 'reference'),
                    'reliability': None,
                    'related': None,
                    'related-target': None,
                    'related-type': None,
                    'specimen': None,
                    'status': None,
                    'subject': ({'Resource': 'Patient',
                                'Path': 'patient'}, 'reference'), 
                    'value-concept': None,
                    'value-date': None,
                    'value-quantity': (['respiratory_rate', 'bpm','diastolic','systolic', 'temperature'], 'quantity'),
                    'value-string': None},
            'gnuhealth.icu.apache2': {'_id': (['id'], 'token'), #Needs to be parsed MODEL-ID-FIELD
                    '_language': None,
                    'date': None,
                    # These values point to keys on the 'fields' dict
                    'name': ({'Respiratory rate': 'rate',
                                'Heart rate': 'pulse',
                                'Temperature': 'temp'}, 'user-defined'),
                    'performer': None,
                    'reliability': None,
                    'related': None,
                    'related-target': None,
                    'related-type': None,
                    'specimen': None,
                    'status': None,
                    'subject': (['name.patient'], 'reference'), 
                    'value-concept': None,
                    'value-date': None,
                    'value-quantity': (['respiratory_rate', 'heart_rate', 'temperature'], 'quantity'),
                    'value-string': None},
            'gnuhealth.patient.rounding': {'_id': (['id'], 'token'), #Needs to be parsed MODEL-ID-FIELD
                    '_language': None,
                    'date': None,
                    # These values point to keys on the 'fields' dict
                    'name': ({'Respiratory rate': 'rate',
                                'Heart rate': 'pulse', 'Systolic pressure': 'press_s',
                                'Diastolic pressure': 'press_d',
                                'Temperature': 'temp'}, 'user-defined'),
                    'performer': (['health_professional'], 'reference'),
                    'reliability': None,
                    'related': None,
                    'related-target': None,
                    'related-type': None,
                    'specimen': None,
                    'status': None,
                    'subject': (['name.patient'], 'reference'), 
                    'value-concept': None,
                    'value-date': None,
                    'value-quantity': (['respiratory_rate', 'bpm','diastolic','systolic', 'temperature'], 'quantity'),
                    'value-string': None},
            'gnuhealth.patient.ambulatory_care': {'_id': (['id'], 'token'), #Needs to be parsed MODEL-ID-FIELD
                    '_language': None,
                    'date': None,
                    # These values point to keys on the 'fields' dict
                    'name': ({'Respiratory rate': 'rate',
                                'Heart rate': 'pulse', 'Systolic pressure': 'press_s',
                                'Diastolic pressure': 'press_d',
                                'Temperature': 'temp'}, 'user-defined'),
                    'performer': (['health_professional'], 'reference'),
                    'reliability': None,
                    'related': None,
                    'related-target': None,
                    'related-type': None,
                    'specimen': None,
                    'status': None,
                    'subject': (['patient'], 'reference'),
                    'value-concept': None,
                    'value-date': None,
                    'value-quantity': (['respiratory_rate', 'bpm','diastolic','systolic', 'temperature'], 'quantity'),
                    'value-string': None}}

#TODO Put restrictions on code values (interp, status, reliability, etc)
class health_Observation(supermod.Observation, Observation_Map, ExportXMLMixin):
    def __init__(self, *args, **kwargs):
        gnu=kwargs.pop('gnu_record', None)
        field=kwargs.pop('field', None)
        super(health_Observation, self).__init__(*args, **kwargs)
        if gnu:
            self.set_gnu_observation(gnu, field=field)

    def set_gnu_observation(self, obs, field=None):
        """Set gnu observation"""
        self.gnu_obs = obs
        self.field = field
        self.model_type = self.gnu_obs.__name__

        # Only certain models
        if self.model_type not in self.model_mapping:
            raise ValueError('Not a valid model')

        self.map_ = self.model_mapping[self.model_type]

        # These models require fields
        if self.map_.get('fields') and self.field is None:
            raise FieldError('This model requires a field')

        # Not these
        if self.map_.get('value') and self.field is not None:
            raise ValueError('Ambiguous field; not required')

        #self.search_prefix = self.url_prefixes[self.model_type]

        if self.field:
            self.model_field = self.map_['fields'][self.field]
            self.description=self.gnu_obs.fields_get(self.model_field)[self.model_field]['string']
        else:
            self.model_field = self.map_['value']
            self.description = self.gnu_obs.name

        # Quietly import the info
        self.__import_from_gnu_observation()
        self.__set_feed_info()

    def create_observation(self, lab_test, units, lab_result, patient):
        """Create observation.

        ***Must be connected to a patient***

        NOTE: Must create singleton Lab Test at the moment(?)
        NOTE: Where to put vital signs? Evals?
        """
        patient_id = find_record(patient, [('id', '=', self.models['patient']['id'])])
        if not patient_id:
            raise ValueError
        units_id = find_record(units, [['OR', [('name', '=', self.models['units']['name'])],
                                            [('code', '=', self.models['units']['code'])]]])
        if not units_id:
            raise ValueError

        pass

    def __set_gnu_models(self):
        pass

    def __set_feed_info(self):
        """Set the feed-relevant data"""
        if self.gnu_obs:
            if RUN_FLASK:
                uri = url_for('obs_record',
                                log_id=self.gnu_obs.id,
                                _external=True)
            else:
                uri = dumb_url_generate(['Observation',
                                self.gnu_obs.id])
            self.feed={'id': uri,
                    'published': self.gnu_obs.create_date,
                    'updated': self.gnu_obs.write_date or self.gnu_obs.create_date,
                    'title': self.identifier.label.value
                        }

    def __import_from_gnu_observation(self):
        """Set the data from the model"""
        if self.gnu_obs:
            self.set_identifier(
                    self.gnu_obs,
                    safe_attrgetter(self.gnu_obs, self.map_['patient']),
                    safe_attrgetter(self.gnu_obs, self.map_['date']))
            self.set_subject(
                    safe_attrgetter(self.gnu_obs,
                                            self.map_['patient']))
            self.set_comments(
                    safe_attrgetter(self.gnu_obs, self.map_['comments']))
            self.set_valueQuantity(
                    safe_attrgetter(self.gnu_obs, self.model_field),
                    safe_attrgetter(self.gnu_obs, 'units.name'))
            self.set_referenceRange(
                    safe_attrgetter(self.gnu_obs, 'units.name', default='unknown'),
                    safe_attrgetter(self.gnu_obs, 'lower_limit'),
                    safe_attrgetter(self.gnu_obs, 'upper_limit'))
            self.set_interpretation(
                    safe_attrgetter(self.gnu_obs, self.model_field),
                    safe_attrgetter(self.gnu_obs, 'lower_limit'),
                    safe_attrgetter(self.gnu_obs, 'upper_limit'))
            self.set_status(
                    safe_attrgetter(self.gnu_obs, self.map_['excluded']),
                    safe_attrgetter(self.gnu_obs, self.model_field))
            self.set_reliability('ok')
            self.set_issued(
                    self.gnu_obs.write_date or self.gnu_obs.create_date)
            self.set_performer(
                    safe_attrgetter(self.gnu_obs, self.map_['performer']))
            self.set_name(
                    self.description)

    def set_comments(self, comments):
        """Extends superclass for convenience

        Set comments

        Keyword arguments:
        comments -- the comments
        """

        if comments:
            m = supermod.string(value=str(comments))
            super(health_Observation, self).set_comments(m)

    def set_identifier(self, observation, patient, date):
        """Extends superclass for convenience

        Set the identifier from the data model

        Keyword arguments:
        observation -- the observation (Health model)
        patient -- patient (Health model)
        date -- date of observation (datetime object)
        """

        if observation and patient:
            if date is not None:
                label = '{0} value for {1} on {2}'.format(observation.name,
                                                    patient.name.rec_name,
                                                    date.strftime('%Y/%m/%d'))
            else:
                label = '{0} value for {1} on unknown'.format(observation.name,
                                                    patient.name.rec_name)

            if RUN_FLASK:
                value = url_for('obs_record', log_id=observation.id)
            else:
                value = dumb_url_generate(['Observation', observation.id])
            ident = supermod.Identifier(
                        label=supermod.string(value=label),
                        value=supermod.string(value=value))
            super(health_Observation, self).set_identifier(ident)

    def set_interpretation(self, value, lower_limit, upper_limit):
        """Extends superclass for convenience

        Set the interpretation

        Keyword arguments:
        value -- observation value
        lower_limit -- the lower normal
        upper_limit -- the upper normal
        """
        # TODO: Interpretation is complicated

        if value and lower_limit and upper_limit:
            interp = supermod.CodeableConcept()
            interp.coding = [supermod.Coding()]
            if value < lower_limit:
                v = 'L'
                d = 'Low'
            elif value > upper_limit:
                v = 'H'
                d = 'High'
            else:
                v = 'N'
                d = 'Normal'
            interp.coding[0].system = supermod.uri(value='http://hl7.org/fhir/v2/0078')
            interp.coding[0].code = supermod.code(value=v)
            interp.coding[0].display = supermod.string(value=d)
            super(health_Observation, self).set_interpretation(interp)

    def set_issued(self, issued):
        """Extends superclass for convenience

        Set newest update time

        Keyword arguments:
        issued -- time issued (datetime object)
        """

        if issued is not None:
            time=issued.strftime("%Y-%m-%dT%H:%M:%S")
            instant = supermod.instant(value=time)
            super(health_Observation, self).set_issued(instant)

    def set_name(self, name):
        """Extends superclass for convenience

        Set the observation type

        Keyword arguments:
        name -- the description of the observation
        """
        #TODO Better coding

        if name:
            n = supermod.CodeableConcept()
            n.coding = [supermod.Coding()]
            n.coding[0].display = supermod.string(value=name)
            super(health_Observation, self).set_name(n)

    def set_performer(self, performer):
        """Extends superclass for convenience

        Set who/what captured the observation

        Keyword arguments:
        performer -- who performed the observation (Health model)
        """

        if performer:
            try:
                if RUN_FLASK:
                    uri=url_for('hp_record', log_id=performer.id)
                else:
                    uri=dumb_url_generate('/Practitioner', str(performer.id))
                display = performer.name.rec_name
                ref=supermod.ResourceReference()
                ref.display = supermod.string(value=display)
                ref.reference = supermod.string(value=uri)
            except:
                # Not absolutely needed, so continue execution
                pass
            else:
                super(health_Observation, self).set_performer([ref])

    def set_referenceRange(self, units, lower_limit, upper_limit):
        """Extends superclass for convenience

        Set reference range from data model

        Keyword arguments:
        units -- units name
        lower_limit -- lower limit
        upper_limit -- upper limit
        """

        if units is not None \
                and lower_limit is not None \
                and upper_limit is not None:
            ref = supermod.Observation_ReferenceRange()
            #ref.age = supermod.Range() #Not relevant, usually
            ref.low = supermod.Quantity()
            ref.high = supermod.Quantity()
            ref.low.units = ref.high.units = supermod.string(value=units)
            ref.low.value = supermod.decimal(value=lower_limit)
            ref.high.value = supermod.decimal(value=upper_limit)
            ref.meaning = supermod.Coding()
            ref.meaning.system = supermod.uri(value='http://hl7.org/fhir/referencerange-meaning')
            ref.meaning.code = supermod.code(value='normal')
            ref.meaning.display = supermod.string(value='Normal range')
            super(health_Observation, self).set_referenceRange([ref])

    def set_reliability(self, reliability='ok'):
        """Extends superclass for convenience

        Set reliability; mandatory

        Keyword arguments:
        reliability -- how reliable the observation
        """

        rel = supermod.ObservationReliability()
        rel.value = reliability or 'ok'
        super(health_Observation, self).set_reliability(rel)

    def set_status(self, excluded=False, value=None):
        """Extends superclass for convenience

        Set status; mandatory

        Keyword arguments:
        excluded -- excluded or not
        value -- the test value (used to determine specific status)
        """

        s = supermod.ObservationStatus()
        if excluded:
            if value is not None:
                s.value = 'cancelled'
            else:
                s.value = 'entered in error'
        else:
            if value is not None:
                s.value = 'final'
            else:
                s.value = 'temporary'
        super(health_Observation, self).set_status(s)

    def set_valueQuantity(self, value, units=None, code=None, system=None):
        """Extends superclass for convenience

        Set actual value of observation

        Keyword arguments:
        value -- the actual value
        units -- units (e.g., g/dL)
        code -- the code (e.g., 'XHEHA1')
        system -- the code system (e.g., 'http://code.system')
        """

        if value:

            # This is to handle multi-field
            #if self.field == 'temp':
                #units = "degrees C"
                #code = "258710007"
                #system = "http://snomed.info/sct"
            #elif self.field == 'press_d':
                #units = "mm[Hg]"
            #elif self.field == 'press_s':
                #units = "mm[Hg]"
            #elif self.field == 'pulse':
                #units = "beats/min"
            #elif self.field == 'rate':
                #units= "breaths/min"


            q = supermod.Quantity()
            q.value = supermod.decimal(value=value)

            if units:
                q.units = supermod.string(value=str(units))
            if code:
                q.code = supermod.code(value=str(code))
            if system:
                q.system = supermod.uri(value=str(system))
            super(health_Observation, self).set_valueQuantity(q)

    def set_subject(self, subject):
        """Extends superclass for convenience

        Set subject (usually patient); mandatory

        Keyword arguments:
        subject -- the subject (Health model)
        """

        if subject:
            try:
                if RUN_FLASK:
                    uri=url_for('pat_record', log_id=subject.id)
                else:
                    uri=dumb_url_generate('/Patient', str(subject.id))
                display = subject.name.rec_name
                ref=supermod.ResourceReference()
                ref.display = supermod.string(value=display)
                ref.reference = supermod.string(value=uri)
            except:
                raise
            else:
                super(health_Observation, self).set_subject(ref)

    def set_applies_date_time(self):
        pass

    def set_applies_period(self):
        pass

    def set_specimen(self):
        pass

    def set_related(self):
        pass

    def set_method(self):
        pass

supermod.Observation.subclass=health_Observation



from .base_classes import *
from .support_functions import *


class MedicationDispense(Resource):
    """Dispensing a medication to a named patient. This includes a
    description of the supply provided and the instructions for
    administering the medication.If the element is present, it must
    have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, status=None, patient=None, dispenser=None, authorizingPrescription=None, dispense=None, substitution=None):
        self.original_tagname_ = None
        super(MedicationDispense, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.identifier = identifier
        self.status = status
        self.patient = patient
        self.dispenser = dispenser
        if authorizingPrescription is None:
            self.authorizingPrescription = []
        else:
            self.authorizingPrescription = authorizingPrescription
        if dispense is None:
            self.dispense = []
        else:
            self.dispense = dispense
        self.substitution = substitution
    def factory(*args_, **kwargs_):
        if MedicationDispense.subclass:
            return MedicationDispense.subclass(*args_, **kwargs_)
        else:
            return MedicationDispense(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_patient(self): return self.patient
    def set_patient(self, patient): self.patient = patient
    def get_dispenser(self): return self.dispenser
    def set_dispenser(self, dispenser): self.dispenser = dispenser
    def get_authorizingPrescription(self): return self.authorizingPrescription
    def set_authorizingPrescription(self, authorizingPrescription): self.authorizingPrescription = authorizingPrescription
    def add_authorizingPrescription(self, value): self.authorizingPrescription.append(value)
    def insert_authorizingPrescription(self, index, value): self.authorizingPrescription[index] = value
    def get_dispense(self): return self.dispense
    def set_dispense(self, dispense): self.dispense = dispense
    def add_dispense(self, value): self.dispense.append(value)
    def insert_dispense(self, index, value): self.dispense[index] = value
    def get_substitution(self): return self.substitution
    def set_substitution(self, substitution): self.substitution = substitution
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.status is not None or
            self.patient is not None or
            self.dispenser is not None or
            self.authorizingPrescription or
            self.dispense or
            self.substitution is not None or
            super(MedicationDispense, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationDispense', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispense')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationDispense', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationDispense'):
        super(MedicationDispense, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispense')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationDispense', fromsubclass_=False, pretty_print=True):
        super(MedicationDispense, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.patient is not None:
            self.patient.export(outfile, level, namespace_, name_='patient', pretty_print=pretty_print)
        if self.dispenser is not None:
            self.dispenser.export(outfile, level, namespace_, name_='dispenser', pretty_print=pretty_print)
        for authorizingPrescription_ in self.authorizingPrescription:
            authorizingPrescription_.export(outfile, level, namespace_, name_='authorizingPrescription', pretty_print=pretty_print)
        for dispense_ in self.dispense:
            dispense_.export(outfile, level, namespace_, name_='dispense', pretty_print=pretty_print)
        if self.substitution is not None:
            self.substitution.export(outfile, level, namespace_, name_='substitution', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationDispense', mapping_=None):
        element = super(MedicationDispense, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.patient is not None:
            patient_ = self.patient
            patient_.to_etree(element, name_='patient', mapping_=mapping_)
        if self.dispenser is not None:
            dispenser_ = self.dispenser
            dispenser_.to_etree(element, name_='dispenser', mapping_=mapping_)
        for authorizingPrescription_ in self.authorizingPrescription:
            authorizingPrescription_.to_etree(element, name_='authorizingPrescription', mapping_=mapping_)
        for dispense_ in self.dispense:
            dispense_.to_etree(element, name_='dispense', mapping_=mapping_)
        if self.substitution is not None:
            substitution_ = self.substitution
            substitution_.to_etree(element, name_='substitution', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationDispense, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'status':
            obj_ = MedicationDispenseStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'patient':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.patient = obj_
            obj_.original_tagname_ = 'patient'
        elif nodeName_ == 'dispenser':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.dispenser = obj_
            obj_.original_tagname_ = 'dispenser'
        elif nodeName_ == 'authorizingPrescription':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.authorizingPrescription.append(obj_)
            obj_.original_tagname_ = 'authorizingPrescription'
        elif nodeName_ == 'dispense':
            obj_ = MedicationDispense_Dispense.factory()
            obj_.build(child_)
            self.dispense.append(obj_)
            obj_.original_tagname_ = 'dispense'
        elif nodeName_ == 'substitution':
            obj_ = MedicationDispense_Substitution.factory()
            obj_.build(child_)
            self.substitution = obj_
            obj_.original_tagname_ = 'substitution'
        super(MedicationDispense, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationDispense


class MedicationDispense_Dispense(BackboneElement):
    """Dispensing a medication to a named patient. This includes a
    description of the supply provided and the instructions for
    administering the medication."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, identifier=None, status=None, type_=None, quantity=None, medication=None, whenPrepared=None, whenHandedOver=None, destination=None, receiver=None, dosage=None):
        self.original_tagname_ = None
        super(MedicationDispense_Dispense, self).__init__(id, extension, modifierExtension, )
        self.identifier = identifier
        self.status = status
        self.type_ = type_
        self.quantity = quantity
        self.medication = medication
        self.whenPrepared = whenPrepared
        self.whenHandedOver = whenHandedOver
        self.destination = destination
        if receiver is None:
            self.receiver = []
        else:
            self.receiver = receiver
        if dosage is None:
            self.dosage = []
        else:
            self.dosage = dosage
    def factory(*args_, **kwargs_):
        if MedicationDispense_Dispense.subclass:
            return MedicationDispense_Dispense.subclass(*args_, **kwargs_)
        else:
            return MedicationDispense_Dispense(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def get_medication(self): return self.medication
    def set_medication(self, medication): self.medication = medication
    def get_whenPrepared(self): return self.whenPrepared
    def set_whenPrepared(self, whenPrepared): self.whenPrepared = whenPrepared
    def get_whenHandedOver(self): return self.whenHandedOver
    def set_whenHandedOver(self, whenHandedOver): self.whenHandedOver = whenHandedOver
    def get_destination(self): return self.destination
    def set_destination(self, destination): self.destination = destination
    def get_receiver(self): return self.receiver
    def set_receiver(self, receiver): self.receiver = receiver
    def add_receiver(self, value): self.receiver.append(value)
    def insert_receiver(self, index, value): self.receiver[index] = value
    def get_dosage(self): return self.dosage
    def set_dosage(self, dosage): self.dosage = dosage
    def add_dosage(self, value): self.dosage.append(value)
    def insert_dosage(self, index, value): self.dosage[index] = value
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.status is not None or
            self.type_ is not None or
            self.quantity is not None or
            self.medication is not None or
            self.whenPrepared is not None or
            self.whenHandedOver is not None or
            self.destination is not None or
            self.receiver or
            self.dosage or
            super(MedicationDispense_Dispense, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationDispense.Dispense', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispense.Dispense')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationDispense.Dispense', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationDispense.Dispense'):
        super(MedicationDispense_Dispense, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispense.Dispense')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationDispense.Dispense', fromsubclass_=False, pretty_print=True):
        super(MedicationDispense_Dispense, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
        if self.medication is not None:
            self.medication.export(outfile, level, namespace_, name_='medication', pretty_print=pretty_print)
        if self.whenPrepared is not None:
            self.whenPrepared.export(outfile, level, namespace_, name_='whenPrepared', pretty_print=pretty_print)
        if self.whenHandedOver is not None:
            self.whenHandedOver.export(outfile, level, namespace_, name_='whenHandedOver', pretty_print=pretty_print)
        if self.destination is not None:
            self.destination.export(outfile, level, namespace_, name_='destination', pretty_print=pretty_print)
        for receiver_ in self.receiver:
            receiver_.export(outfile, level, namespace_, name_='receiver', pretty_print=pretty_print)
        for dosage_ in self.dosage:
            dosage_.export(outfile, level, namespace_, name_='dosage', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationDispense.Dispense', mapping_=None):
        element = super(MedicationDispense_Dispense, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        if self.medication is not None:
            medication_ = self.medication
            medication_.to_etree(element, name_='medication', mapping_=mapping_)
        if self.whenPrepared is not None:
            whenPrepared_ = self.whenPrepared
            whenPrepared_.to_etree(element, name_='whenPrepared', mapping_=mapping_)
        if self.whenHandedOver is not None:
            whenHandedOver_ = self.whenHandedOver
            whenHandedOver_.to_etree(element, name_='whenHandedOver', mapping_=mapping_)
        if self.destination is not None:
            destination_ = self.destination
            destination_.to_etree(element, name_='destination', mapping_=mapping_)
        for receiver_ in self.receiver:
            receiver_.to_etree(element, name_='receiver', mapping_=mapping_)
        for dosage_ in self.dosage:
            dosage_.to_etree(element, name_='dosage', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationDispense_Dispense, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'status':
            obj_ = MedicationDispenseStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'quantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        elif nodeName_ == 'medication':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.medication = obj_
            obj_.original_tagname_ = 'medication'
        elif nodeName_ == 'whenPrepared':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.whenPrepared = obj_
            obj_.original_tagname_ = 'whenPrepared'
        elif nodeName_ == 'whenHandedOver':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.whenHandedOver = obj_
            obj_.original_tagname_ = 'whenHandedOver'
        elif nodeName_ == 'destination':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.destination = obj_
            obj_.original_tagname_ = 'destination'
        elif nodeName_ == 'receiver':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.receiver.append(obj_)
            obj_.original_tagname_ = 'receiver'
        elif nodeName_ == 'dosage':
            obj_ = MedicationDispense_Dosage.factory()
            obj_.build(child_)
            self.dosage.append(obj_)
            obj_.original_tagname_ = 'dosage'
        super(MedicationDispense_Dispense, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationDispense_Dispense


class MedicationDispense_Dosage(BackboneElement):
    """Dispensing a medication to a named patient. This includes a
    description of the supply provided and the instructions for
    administering the medication.The timing schedule for giving the
    medication to the patient. The Schedule data type allows many
    different expressions, for example. "Every 8 hours"; "Three
    times a day"; "1/2 an hour before breakfast for 10 days from
    23-Dec 2011:"; "15 Oct 2013, 17 Oct 2013 and 1 Nov 2013".If set
    to true or if specified as a CodeableConcept, indicates that the
    medication is only taken when needed within the specified
    schedule rather than at every scheduled dose. If a
    CodeableConcept is present, it indicates the pre-condition for
    taking the Medication."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, additionalInstructions=None, timingDateTime=None, timingPeriod=None, timingSchedule=None, asNeededBoolean=None, asNeededCodeableConcept=None, site=None, route=None, method=None, quantity=None, rate=None, maxDosePerPeriod=None):
        self.original_tagname_ = None
        super(MedicationDispense_Dosage, self).__init__(id, extension, modifierExtension, )
        self.additionalInstructions = additionalInstructions
        self.timingDateTime = timingDateTime
        self.timingPeriod = timingPeriod
        self.timingSchedule = timingSchedule
        self.asNeededBoolean = asNeededBoolean
        self.asNeededCodeableConcept = asNeededCodeableConcept
        self.site = site
        self.route = route
        self.method = method
        self.quantity = quantity
        self.rate = rate
        self.maxDosePerPeriod = maxDosePerPeriod
    def factory(*args_, **kwargs_):
        if MedicationDispense_Dosage.subclass:
            return MedicationDispense_Dosage.subclass(*args_, **kwargs_)
        else:
            return MedicationDispense_Dosage(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_additionalInstructions(self): return self.additionalInstructions
    def set_additionalInstructions(self, additionalInstructions): self.additionalInstructions = additionalInstructions
    def get_timingDateTime(self): return self.timingDateTime
    def set_timingDateTime(self, timingDateTime): self.timingDateTime = timingDateTime
    def get_timingPeriod(self): return self.timingPeriod
    def set_timingPeriod(self, timingPeriod): self.timingPeriod = timingPeriod
    def get_timingSchedule(self): return self.timingSchedule
    def set_timingSchedule(self, timingSchedule): self.timingSchedule = timingSchedule
    def get_asNeededBoolean(self): return self.asNeededBoolean
    def set_asNeededBoolean(self, asNeededBoolean): self.asNeededBoolean = asNeededBoolean
    def get_asNeededCodeableConcept(self): return self.asNeededCodeableConcept
    def set_asNeededCodeableConcept(self, asNeededCodeableConcept): self.asNeededCodeableConcept = asNeededCodeableConcept
    def get_site(self): return self.site
    def set_site(self, site): self.site = site
    def get_route(self): return self.route
    def set_route(self, route): self.route = route
    def get_method(self): return self.method
    def set_method(self, method): self.method = method
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def get_rate(self): return self.rate
    def set_rate(self, rate): self.rate = rate
    def get_maxDosePerPeriod(self): return self.maxDosePerPeriod
    def set_maxDosePerPeriod(self, maxDosePerPeriod): self.maxDosePerPeriod = maxDosePerPeriod
    def hasContent_(self):
        if (
            self.additionalInstructions is not None or
            self.timingDateTime is not None or
            self.timingPeriod is not None or
            self.timingSchedule is not None or
            self.asNeededBoolean is not None or
            self.asNeededCodeableConcept is not None or
            self.site is not None or
            self.route is not None or
            self.method is not None or
            self.quantity is not None or
            self.rate is not None or
            self.maxDosePerPeriod is not None or
            super(MedicationDispense_Dosage, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationDispense.Dosage', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispense.Dosage')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationDispense.Dosage', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationDispense.Dosage'):
        super(MedicationDispense_Dosage, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispense.Dosage')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationDispense.Dosage', fromsubclass_=False, pretty_print=True):
        super(MedicationDispense_Dosage, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.additionalInstructions is not None:
            self.additionalInstructions.export(outfile, level, namespace_, name_='additionalInstructions', pretty_print=pretty_print)
        if self.timingDateTime is not None:
            self.timingDateTime.export(outfile, level, namespace_, name_='timingDateTime', pretty_print=pretty_print)
        if self.timingPeriod is not None:
            self.timingPeriod.export(outfile, level, namespace_, name_='timingPeriod', pretty_print=pretty_print)
        if self.timingSchedule is not None:
            self.timingSchedule.export(outfile, level, namespace_, name_='timingSchedule', pretty_print=pretty_print)
        if self.asNeededBoolean is not None:
            self.asNeededBoolean.export(outfile, level, namespace_, name_='asNeededBoolean', pretty_print=pretty_print)
        if self.asNeededCodeableConcept is not None:
            self.asNeededCodeableConcept.export(outfile, level, namespace_, name_='asNeededCodeableConcept', pretty_print=pretty_print)
        if self.site is not None:
            self.site.export(outfile, level, namespace_, name_='site', pretty_print=pretty_print)
        if self.route is not None:
            self.route.export(outfile, level, namespace_, name_='route', pretty_print=pretty_print)
        if self.method is not None:
            self.method.export(outfile, level, namespace_, name_='method', pretty_print=pretty_print)
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
        if self.rate is not None:
            self.rate.export(outfile, level, namespace_, name_='rate', pretty_print=pretty_print)
        if self.maxDosePerPeriod is not None:
            self.maxDosePerPeriod.export(outfile, level, namespace_, name_='maxDosePerPeriod', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationDispense.Dosage', mapping_=None):
        element = super(MedicationDispense_Dosage, self).to_etree(parent_element, name_, mapping_)
        if self.additionalInstructions is not None:
            additionalInstructions_ = self.additionalInstructions
            additionalInstructions_.to_etree(element, name_='additionalInstructions', mapping_=mapping_)
        if self.timingDateTime is not None:
            timingDateTime_ = self.timingDateTime
            timingDateTime_.to_etree(element, name_='timingDateTime', mapping_=mapping_)
        if self.timingPeriod is not None:
            timingPeriod_ = self.timingPeriod
            timingPeriod_.to_etree(element, name_='timingPeriod', mapping_=mapping_)
        if self.timingSchedule is not None:
            timingSchedule_ = self.timingSchedule
            timingSchedule_.to_etree(element, name_='timingSchedule', mapping_=mapping_)
        if self.asNeededBoolean is not None:
            asNeededBoolean_ = self.asNeededBoolean
            asNeededBoolean_.to_etree(element, name_='asNeededBoolean', mapping_=mapping_)
        if self.asNeededCodeableConcept is not None:
            asNeededCodeableConcept_ = self.asNeededCodeableConcept
            asNeededCodeableConcept_.to_etree(element, name_='asNeededCodeableConcept', mapping_=mapping_)
        if self.site is not None:
            site_ = self.site
            site_.to_etree(element, name_='site', mapping_=mapping_)
        if self.route is not None:
            route_ = self.route
            route_.to_etree(element, name_='route', mapping_=mapping_)
        if self.method is not None:
            method_ = self.method
            method_.to_etree(element, name_='method', mapping_=mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        if self.rate is not None:
            rate_ = self.rate
            rate_.to_etree(element, name_='rate', mapping_=mapping_)
        if self.maxDosePerPeriod is not None:
            maxDosePerPeriod_ = self.maxDosePerPeriod
            maxDosePerPeriod_.to_etree(element, name_='maxDosePerPeriod', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationDispense_Dosage, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'additionalInstructions':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.additionalInstructions = obj_
            obj_.original_tagname_ = 'additionalInstructions'
        elif nodeName_ == 'timingDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.timingDateTime = obj_
            obj_.original_tagname_ = 'timingDateTime'
        elif nodeName_ == 'timingPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.timingPeriod = obj_
            obj_.original_tagname_ = 'timingPeriod'
        elif nodeName_ == 'timingSchedule':
            obj_ = Schedule.factory()
            obj_.build(child_)
            self.timingSchedule = obj_
            obj_.original_tagname_ = 'timingSchedule'
        elif nodeName_ == 'asNeededBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.asNeededBoolean = obj_
            obj_.original_tagname_ = 'asNeededBoolean'
        elif nodeName_ == 'asNeededCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.asNeededCodeableConcept = obj_
            obj_.original_tagname_ = 'asNeededCodeableConcept'
        elif nodeName_ == 'site':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.site = obj_
            obj_.original_tagname_ = 'site'
        elif nodeName_ == 'route':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.route = obj_
            obj_.original_tagname_ = 'route'
        elif nodeName_ == 'method':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.method = obj_
            obj_.original_tagname_ = 'method'
        elif nodeName_ == 'quantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        elif nodeName_ == 'rate':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.rate = obj_
            obj_.original_tagname_ = 'rate'
        elif nodeName_ == 'maxDosePerPeriod':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.maxDosePerPeriod = obj_
            obj_.original_tagname_ = 'maxDosePerPeriod'
        super(MedicationDispense_Dosage, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationDispense_Dosage


class MedicationDispense_Substitution(BackboneElement):
    """Dispensing a medication to a named patient. This includes a
    description of the supply provided and the instructions for
    administering the medication."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, type_=None, reason=None, responsibleParty=None):
        self.original_tagname_ = None
        super(MedicationDispense_Substitution, self).__init__(id, extension, modifierExtension, )
        self.type_ = type_
        if reason is None:
            self.reason = []
        else:
            self.reason = reason
        if responsibleParty is None:
            self.responsibleParty = []
        else:
            self.responsibleParty = responsibleParty
    def factory(*args_, **kwargs_):
        if MedicationDispense_Substitution.subclass:
            return MedicationDispense_Substitution.subclass(*args_, **kwargs_)
        else:
            return MedicationDispense_Substitution(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_reason(self): return self.reason
    def set_reason(self, reason): self.reason = reason
    def add_reason(self, value): self.reason.append(value)
    def insert_reason(self, index, value): self.reason[index] = value
    def get_responsibleParty(self): return self.responsibleParty
    def set_responsibleParty(self, responsibleParty): self.responsibleParty = responsibleParty
    def add_responsibleParty(self, value): self.responsibleParty.append(value)
    def insert_responsibleParty(self, index, value): self.responsibleParty[index] = value
    def hasContent_(self):
        if (
            self.type_ is not None or
            self.reason or
            self.responsibleParty or
            super(MedicationDispense_Substitution, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationDispense.Substitution', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispense.Substitution')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationDispense.Substitution', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationDispense.Substitution'):
        super(MedicationDispense_Substitution, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispense.Substitution')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationDispense.Substitution', fromsubclass_=False, pretty_print=True):
        super(MedicationDispense_Substitution, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        for reason_ in self.reason:
            reason_.export(outfile, level, namespace_, name_='reason', pretty_print=pretty_print)
        for responsibleParty_ in self.responsibleParty:
            responsibleParty_.export(outfile, level, namespace_, name_='responsibleParty', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationDispense.Substitution', mapping_=None):
        element = super(MedicationDispense_Substitution, self).to_etree(parent_element, name_, mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        for reason_ in self.reason:
            reason_.to_etree(element, name_='reason', mapping_=mapping_)
        for responsibleParty_ in self.responsibleParty:
            responsibleParty_.to_etree(element, name_='responsibleParty', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationDispense_Substitution, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'reason':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.reason.append(obj_)
            obj_.original_tagname_ = 'reason'
        elif nodeName_ == 'responsibleParty':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.responsibleParty.append(obj_)
            obj_.original_tagname_ = 'responsibleParty'
        super(MedicationDispense_Substitution, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationDispense_Substitution


class MedicationDispenseStatus(Element):
    """A code specifying the state of the dispense event.If the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(MedicationDispenseStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if MedicationDispenseStatus.subclass:
            return MedicationDispenseStatus.subclass(*args_, **kwargs_)
        else:
            return MedicationDispenseStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(MedicationDispenseStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationDispenseStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispenseStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationDispenseStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationDispenseStatus'):
        super(MedicationDispenseStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationDispenseStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationDispenseStatus', fromsubclass_=False, pretty_print=True):
        super(MedicationDispenseStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationDispenseStatus', mapping_=None):
        element = super(MedicationDispenseStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(MedicationDispenseStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(MedicationDispenseStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class MedicationDispenseStatus



import xml.dom.minidom
domimpl = xml.dom.minidom.getDOMImplementation()

import logging
import urllib.request, urllib.parse, urllib.error
import urllib.parse

from . import utils
from .constants import RT_ALLPROP, RT_PROPNAME, RT_PROP
from .errors import DAV_Error, DAV_NotFound

log = logging.getLogger(__name__)


class PROPFIND:
    """ parse a propfind xml element and extract props

    It will set the following instance vars:

    request_class   : ALLPROP | PROPNAME | PROP
    proplist    : list of properties
    nsmap       : map of namespaces

    The list of properties will contain tuples of the form
    (element name, ns_prefix, ns_uri)


    """

    def __init__(self, uri, dataclass, depth, body):
        self.request_type = None
        self.nsmap = {}
        self.proplist = {}
        self.default_ns = None
        self._dataclass = dataclass
        self._depth = str(depth)
        self._uri = uri.rstrip(b'/')
        self._has_body = None   # did we parse a body?

        if dataclass.verbose:
            log.info('PROPFIND: Depth is %s, URI is %s' % (depth, uri))

        if body:
            self.request_type, self.proplist, self.namespaces = \
                utils.parse_propfind(body)
            self._has_body = True

    def createResponse(self):
        """ Create the multistatus response

        This will be delegated to the specific method
        depending on which request (allprop, propname, prop)
        was found.

        If we get a PROPNAME then we simply return the list with empty
        values which we get from the interface class

        If we get an ALLPROP we first get the list of properties and then
        we do the same as with a PROP method.

        """

        # check if resource exists
        if not self._dataclass.exists(self._uri):
            raise DAV_NotFound

        df = None
        if self.request_type == RT_ALLPROP:
            df = self.create_allprop()

        if self.request_type == RT_PROPNAME:
            df = self.create_propname()

        if self.request_type == RT_PROP:
            df = self.create_prop()

        if df is not None:
            return df

        # no body means ALLPROP!
        df = self.create_allprop()
        return df

    def create_propname(self):
        """ create a multistatus response for the prop names """

        dc = self._dataclass
        # create the document generator
        doc = domimpl.createDocument(None, "multistatus", None)
        ms = doc.documentElement
        ms.setAttribute("xmlns:D", "DAV:")
        ms.tagName = 'D:multistatus'

        if self._depth == "0":
            pnames = dc.get_propnames(self._uri)
            re = self.mk_propname_response(self._uri, pnames, doc)
            ms.appendChild(re)

        elif self._depth == "1":
            pnames = dc.get_propnames(self._uri)
            re = self.mk_propname_response(self._uri, pnames, doc)
            ms.appendChild(re)

            for newuri in dc.get_childs(self._uri):
                pnames = dc.get_propnames(newuri)
                re = self.mk_propname_response(newuri, pnames, doc)
                ms.appendChild(re)
        elif self._depth == 'infinity':
            uri_list = [self._uri]
            while uri_list:
                uri = uri_list.pop()
                pnames = dc.get_propnames(uri)
                re = self.mk_propname_response(uri, pnames, doc)
                ms.appendChild(re)
                uri_childs = self._dataclass.get_childs(uri)
                if uri_childs:
                    uri_list.extend(uri_childs)

        return doc.toxml(encoding="utf-8")

    def create_allprop(self):
        """ return a list of all properties """
        self.proplist = {}
        self.namespaces = []
        for ns, plist in list(self._dataclass.get_propnames(self._uri).items()):
            self.proplist[ns] = plist
            self.namespaces.append(ns)

        return self.create_prop()

    def create_prop(self):
        """ handle a <prop> request

        This will

        1. set up the <multistatus>-Framework

        2. read the property values for each URI
           (which is dependant on the Depth header)
           This is done by the get_propvalues() method.

        3. For each URI call the append_result() method
           to append the actual <result>-Tag to the result
           document.

        We differ between "good" properties, which have been
        assigned a value by the interface class and "bad"
        properties, which resulted in an error, either 404
        (Not Found) or 403 (Forbidden).

        """
        # create the document generator
        doc = domimpl.createDocument(None, "multistatus", None)
        ms = doc.documentElement
        ms.setAttribute("xmlns:D", "DAV:")
        ms.tagName = 'D:multistatus'

        if self._depth == "0":
            gp, bp = self.get_propvalues(self._uri)
            res = self.mk_prop_response(self._uri, gp, bp, doc)
            ms.appendChild(res)

        elif self._depth == "1":
            gp, bp = self.get_propvalues(self._uri)
            res = self.mk_prop_response(self._uri, gp, bp, doc)
            ms.appendChild(res)

            for newuri in self._dataclass.get_childs(self._uri):
                gp, bp = self.get_propvalues(newuri)
                res = self.mk_prop_response(newuri, gp, bp, doc)
                ms.appendChild(res)
        elif self._depth == 'infinity':
            uri_list = [self._uri]
            while uri_list:
                uri = uri_list.pop()
                gp, bp = self.get_propvalues(uri)
                res = self.mk_prop_response(uri, gp, bp, doc)
                ms.appendChild(res)
                uri_childs = self._dataclass.get_childs(uri)
                if uri_childs:
                    uri_list.extend(uri_childs)

        return doc.toxml(encoding="utf-8")

    def mk_propname_response(self, uri, propnames, doc):
        """ make a new <prop> result element for a PROPNAME request

        This will simply format the propnames list.
        propnames should have the format {NS1 : [prop1, prop2, ...], NS2: ...}

        """
        re = doc.createElement("D:response")

        if self._dataclass.baseurl:
            uri = self._dataclass.baseurl + '/' + '/'.join(uri.split('/')[3:])

        # write href information
        uparts = urllib.parse.urlparse(uri)
        fileloc = uparts[2]
        href = doc.createElement("D:href")

        huri = doc.createTextNode(uparts[0] + '://' +
                                  '/'.join(uparts[1:2]) +
                                  urllib.parse.quote(fileloc))
        href.appendChild(huri)
        re.appendChild(href)

        ps = doc.createElement("D:propstat")
        nsnum = 0

        for ns, plist in list(propnames.items()):
            # write prop element
            pr = doc.createElement("D:prop")
            nsp = "ns" + str(nsnum)
            pr.setAttribute("xmlns:" + nsp, ns)
            nsnum += 1

            # write propertynames
            for p in plist:
                pe = doc.createElement(nsp + ":" + p)
                pr.appendChild(pe)

            ps.appendChild(pr)
        re.appendChild(ps)

        return re

    def mk_prop_response(self, uri, good_props, bad_props, doc):
        """ make a new <prop> result element

        We differ between the good props and the bad ones for
        each generating an extra <propstat>-Node (for each error
        one, that means).

        """
        re = doc.createElement("D:response")
        # append namespaces to response
        nsnum = 0
        for nsname in self.namespaces:
            if nsname != 'DAV:':
                re.setAttribute("xmlns:ns" + str(nsnum), nsname)
            nsnum += 1

        if self._dataclass.baseurl:
            uri = self._dataclass.baseurl + '/' + '/'.join(uri.split('/')[3:])

        # write href information
        if not isinstance(uri, str):
            uparts = urllib.parse.urlparse(uri.decode())
        else:
            uparts = urllib.parse.urlparse(uri)

        fileloc = uparts[2]
        href = doc.createElement("D:href")

        huri = doc.createTextNode(uparts[0] + '://' +
                                  '/'.join(uparts[1:2]) +
                                  urllib.parse.quote(fileloc))
        href.appendChild(huri)
        re.appendChild(href)

        # write good properties
        ps = doc.createElement("D:propstat")
        if good_props:
            re.appendChild(ps)

        gp = doc.createElement("D:prop")
        for ns in list(good_props.keys()):
            if ns != 'DAV:':
                ns_prefix = "ns" + str(self.namespaces.index(ns)) + ":"
            else:
                ns_prefix = 'D:'
            for p, v in list(good_props[ns].items()):

                pe = doc.createElement(ns_prefix + str(p))
                if isinstance(v, xml.dom.minidom.Element):
                    pe.appendChild(v)
                elif isinstance(v, list):
                    for val in v:
                        pe.appendChild(val)
                else:
                    if p == "resourcetype":
                        if v == 1:
                            ve = doc.createElement("D:collection")
                            pe.appendChild(ve)
                    else:
                        ve = doc.createTextNode(v)
                        pe.appendChild(ve)

                gp.appendChild(pe)

        ps.appendChild(gp)
        s = doc.createElement("D:status")
        t = doc.createTextNode("HTTP/1.1 200 OK")
        s.appendChild(t)
        ps.appendChild(s)
        re.appendChild(ps)

        # now write the errors!
        if len(list(bad_props.items())):

            # write a propstat for each error code
            for ecode in list(bad_props.keys()):
                ps = doc.createElement("D:propstat")
                re.appendChild(ps)
                bp = doc.createElement("D:prop")
                ps.appendChild(bp)

                for ns in list(bad_props[ecode].keys()):
                    if ns != 'DAV:':
                        ns_prefix = "ns" + str(self.namespaces.index(ns)) + ":"
                    else:
                        ns_prefix = 'D:'

                    for p in bad_props[ecode][ns]:
                        pe = doc.createElement(ns_prefix + str(p))
                        bp.appendChild(pe)

                s = doc.createElement("D:status")
                t = doc.createTextNode(utils.gen_estring(ecode))
                s.appendChild(t)
                ps.appendChild(s)
                re.appendChild(ps)

        # return the new response element
        return re

    def get_propvalues(self, uri):
        """ create lists of property values for an URI

        We create two lists for an URI: the properties for
        which we found a value and the ones for which we
        only got an error, either because they haven't been
        found or the user is not allowed to read them.

        """
        good_props = {}
        bad_props = {}

        ddc = self._dataclass
        for ns, plist in list(self.proplist.items()):
            good_props[ns] = {}
            for prop in plist:
                ec = 0
                try:
                    r = ddc.get_prop(uri, ns, prop)
                    good_props[ns][prop] = r
                except DAV_Error as error_code:
                    ec = error_code.args[0]

                # ignore props with error_code if 0 (invisible)
                if ec == 0:
                    continue

                if ec in bad_props:
                    if ns in bad_props[ec]:
                        bad_props[ec][ns].append(prop)
                    else:
                        bad_props[ec][ns] = [prop]
                else:
                    bad_props[ec] = {ns: [prop]}

        return good_props, bad_props



from werkzeug.routing import BaseConverter, ValidationError
from operator import attrgetter
from flask import make_response, g
from StringIO import StringIO

def get_address(string):
    '''Given string, retrieve full address, easily parsed'''
    import requests
    ENDPOINT_URL='https://open.mapquestapi.com/nominatim/v1/search'
    resp = requests.get(ENDPOINT_URL, params={'q':str(string),
                                            'format': 'json',
                                            'addressdetails': 1,
                                            'limit': 1})
    details = resp.json()
    if details:
        ad = details[0].get('address', {})
        try:
            ad['lat']=float(details[0].get('lat'))
        except:
            pass
        try:
            ad['lon']=float(details[0].get('lon'))
        except:
            pass
        try:
            ad['house_number']=int(ad['house_number'])
        except:
            pass

        return ad

class recordConverter(BaseConverter):
    '''Handle Model-ID-Field endpoint values'''
    def to_python(self, value):
        tmp = [None, None, None]
        for k,v in enumerate(value.split('-')):
            if k == 3: raise ValidationError()
            tmp[k]=v
        try:
            tmp[1]=int(tmp[1])
        except:
            raise ValidationError()
        return tmp

    def to_url(self, values):
        return '-'.join([str(value) for value in values if value is not None])

def search_error_string(args):
    """Returns string for no matches with given arguments
        (used by search interactions)

        params:
            args ===> request.args object
        returns:
            string
    """
    st =[]
    for k,v in args.items():
        st.append(': '.join([k,v]))
    return 'No matching record(s) for {0}'.format('\n'.join(st) or '/')

def output_xml(data, code, headers=None):
    """Output response to xml

    Data could be dict, fhir class, string, etc.
    Code is code status of response
    Headers is dict of headers
    """
    if hasattr(data, 'export_to_xml_string'):
        resp = make_response(data.export_to_xml_string(), code)
    elif hasattr(data, 'export'):
        output=StringIO()
        data.export(outfile=output, namespacedef_='xmlns="http://hl7.org/fhir"', pretty_print=False, level=4)
        content = output.getvalue()
        output.close()
        resp = make_response(content, code)
    else:
        if isinstance(data, dict):
            #TODO Fix this to use Flask-Restful error catching
            #Check for error from Flask-Restful
            msg = data.get('message', 'Unknown return')
        elif isinstance(data, basestring):
            msg = data
        else:
            msg = 'Unknown return'
        resp = make_response(msg, code)
    resp.headers.extend(headers or {})
    #resp.headers['Content-type']='application/xml+fhir'
    return resp

def get_userid():
    """Return g.user.id, or None"""
    try:
        return int(g.user.id)
    except:
        return None

#def safe_attrgetter(obj, *attrs, default=None): #py3 declaration
def safe_attrgetter(obj, *attrs, **kwargs):
    """The fail-safe version of attrgetter"""
    default = kwargs.get('default', None) #py2 compatibility
    v = []
    for attr in attrs:
        try:
           x=attrgetter(attr)(obj)
        except:
           x=default
        v.append(x)
    if len(v) == 1:
        return v[0]
    return v

__all__=['get_address', 'recordConverter', 'search_error_string',
                'output_xml', 'get_userid', 'safe_attrgetter']



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields, Unique
from trytond.transaction import Transaction
from trytond.pyson import Eval, Not, Bool, PYSONEncoder, Equal, And, Or, If
from trytond import backend


__all__ = ['DiseaseGene', 'ProteinDisease', 'GeneVariant','GeneVariantPhenotype',
            'PatientGeneticRisk', 'FamilyDiseases', 'GnuHealthPatient']


class DiseaseGene(ModelSQL, ModelView):
    'Disease Genes'
    __name__ = 'gnuhealth.disease.gene'

    name = fields.Char('Gene Name', required=True,select=True)
    protein_name = fields.Char('Protein Code', 
        help="Encoding Protein Code, such as UniProt protein name", 
        select=True)
    long_name = fields.Char('Official Long Name', translate=True)
    gene_id = fields.Char('Gene ID',
        help="default code from NCBI Entrez database.", select=True)
    chromosome = fields.Char('Chromosome',
        help="Name of the affected chromosome", select=True)
    location = fields.Char('Location', help="Locus of the chromosome")

    info = fields.Text('Information', help="Extra Information")
    variants = fields.One2Many('gnuhealth.gene.variant', 'name',
     'Variants')
        
    protein_uri = fields.Function(fields.Char("Protein URI"),
     'get_protein_uri')
    
    def get_protein_uri(self, name):
        ret_url=''
        if (self.protein_name):
            ret_url = 'http://www.uniprot.org/uniprot/' + \
                str(self.protein_name)
        return ret_url

    @classmethod
    def __setup__(cls):
        super(DiseaseGene, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('name_unique', Unique(t,t.name),
                'The Official Symbol name must be unique'),
            ]

    def get_rec_name(self, name):
        return self.name + ':' + self.long_name

    @classmethod
    def search_rec_name(cls, name, clause):
        # Search for the official and long name
        field = None
        for field in ('name', 'long_name'):
            parties = cls.search([(field,) + tuple(clause[1:])], limit=1)
            if parties:
                break
        if parties:
            return [(field,) + tuple(clause[1:])]
        return [(cls._rec_name,) + tuple(clause[1:])]

    @classmethod
    # Update to version 3.2
    def __register__(cls, module_name):
        super(DiseaseGene, cls).__register__(module_name)

        cursor = Transaction().connection.cursor()
        TableHandler = backend.get('TableHandler')
        table = TableHandler(cls, module_name)
        # Insert the current "specialty" associated to the HP in the
        # table that keeps the specialties associated to different health
        # professionals, gnuhealth.hp_specialty

        if table.column_exist('dominance'):
            # Drop old specialty column
            # which is now part of the gene variant phenotype
            table.drop_column('dominance')


class ProteinDisease(ModelSQL, ModelView):
    'Protein related disorders'
    __name__ = 'gnuhealth.protein.disease'

    name = fields.Char('Disease', required=True,select=True,
        help="Uniprot Disease Code")
        
    disease_name = fields.Char('Disease name', translate=True)
    acronym = fields.Char('Acronym', required=True,select=True,
        help="Disease acronym / mnemonics")

    disease_uri = fields.Function(fields.Char("Disease URI"),
     'get_disease_uri')

    mim_reference = fields.Char('MIM', help="MIM -Mendelian Inheritance in Man- DB reference)")

    gene_variant = fields.One2Many('gnuhealth.gene.variant.phenotype', 'phenotype',
        'Gene Variants', help="Gene variants involved in this condition")
        
    dominance = fields.Selection([
        (None, ''),
        ('d', 'dominant'),
        ('r', 'recessive'),
        ('c', 'codominance'),
        ], 'Dominance', select=True)


    description = fields.Text('Description')        
    

    def get_disease_uri(self, name):
        ret_url=''
        if (self.name):
            ret_url = 'http://www.uniprot.org/diseases/' + \
                str(self.name)
        return ret_url

    @classmethod
    def __setup__(cls):
        super(ProteinDisease, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('name_unique', Unique(t,t.name),
                'The Disease Code  name must be unique'),
            ]

    def get_rec_name(self, name):
        return self.name + ':' + self.disease_name

    @classmethod
    def search_rec_name(cls, name, clause):
        # Search for the disease code or disease name
        field = None
        for field in ('name', 'disease_name'):
            parties = cls.search([(field,) + tuple(clause[1:])], limit=1)
            if parties:
                break
        if parties:
            return [(field,) + tuple(clause[1:])]
        return [(cls._rec_name,) + tuple(clause[1:])]

class GeneVariant(ModelSQL, ModelView):
    'Gene Sequence Variant'
    __name__ = 'gnuhealth.gene.variant'

    name = fields.Many2One('gnuhealth.disease.gene', 'Gene',
        required=True)
    variant = fields.Char("Variant", required=True, select=True)
    aa_change = fields.Char('Change', help="Resulting amino acid change")
    phenotypes = fields.One2Many('gnuhealth.gene.variant.phenotype', 'variant',
     'Phenotypes')
        
    @classmethod
    def __setup__(cls):
        super(GeneVariant, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('variant_unique', Unique(t,t.variant),
                'The variant ID must be unique'),
            ('aa_unique', Unique(t,t.variant,t.aa_change),
                'The resulting AA change for this gene already exists'),
            ]

    def get_rec_name(self, name):
        return self.name.rec_name + " : " + self.variant + " : " + self.aa_change


class GeneVariantPhenotype(ModelSQL, ModelView):
    'Gene Sequence Variant Phenotypes'
    __name__ = 'gnuhealth.gene.variant.phenotype'

    name = fields.Char('Code', required=True)
    variant = fields.Many2One('gnuhealth.gene.variant', 'Variant',
        required=True)
 
    gene = fields.Function(fields.Many2One(
        'gnuhealth.disease.gene', 'Gene',
        depends=['variant']),'get_gene',
        searcher='search_gene')

    phenotype = fields.Many2One('gnuhealth.protein.disease', 'Phenotype',
        required=True)


    def get_gene(self, name):
        if (self.variant):
            return self.variant.name.id


    def get_rec_name(self, name):
        if self.phenotype:
            return self.phenotype.rec_name

    @classmethod
    def search_rec_name(cls, name, clause):
        return [('phenotype',) + tuple(clause[1:])]

    @classmethod
    def search_gene(cls, name, clause):
        res = []
        value = clause[2]
        res.append(('variant.name', clause[1], value))
        return res

    @classmethod
    def __setup__(cls):
        super(GeneVariantPhenotype, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('code', Unique(t,t.name),
                'This code already exists'),
            ]


class PatientGeneticRisk(ModelSQL, ModelView):
    'Patient Genetic Information'
    __name__ = 'gnuhealth.patient.genetic.risk'

    patient = fields.Many2One('gnuhealth.patient', 'Patient', select=True)
    disease_gene = fields.Many2One('gnuhealth.disease.gene',
        'Disease Gene', required=True)
    natural_variant = fields.Many2One('gnuhealth.gene.variant', 'Variant',
        domain=[('name', '=', Eval('disease_gene'))],
        depends=['disease_gene'])
    variant_phenotype = fields.Many2One('gnuhealth.gene.variant.phenotype',\
        'Phenotype',
        domain=[('variant', '=', Eval('natural_variant'))],
        depends=['natural_variant'])
    onset = fields.Integer('Onset', help="Age in years")
    
    notes = fields.Char("Notes")

class FamilyDiseases(ModelSQL, ModelView):
    'Family Diseases'
    __name__ = 'gnuhealth.patient.family.diseases'

    patient = fields.Many2One('gnuhealth.patient', 'Patient', select=True)
    name = fields.Many2One('gnuhealth.pathology', 'Condition', required=True)
    xory = fields.Selection([
        (None, ''),
        ('m', 'Maternal'),
        ('f', 'Paternal'),
        ('s', 'Sibling'),
        ], 'Maternal or Paternal', select=True)

    relative = fields.Selection([
        ('mother', 'Mother'),
        ('father', 'Father'),
        ('brother', 'Brother'),
        ('sister', 'Sister'),
        ('aunt', 'Aunt'),
        ('uncle', 'Uncle'),
        ('nephew', 'Nephew'),
        ('niece', 'Niece'),
        ('grandfather', 'Grandfather'),
        ('grandmother', 'Grandmother'),
        ('cousin', 'Cousin'),
        ], 'Relative',
        help='First degree = siblings, mother and father\n'
            'Second degree = Uncles, nephews and Nieces\n'
            'Third degree = Grandparents and cousins',
        required=True)


class GnuHealthPatient (ModelSQL, ModelView):
    'Add to the Medical patient_data class (gnuhealth.patient) the genetic ' \
    'and family risks'
    __name__ = 'gnuhealth.patient'

    genetic_risks = fields.One2Many('gnuhealth.patient.genetic.risk',
        'patient', 'Genetic Information')
    family_history = fields.One2Many('gnuhealth.patient.family.diseases',
        'patient', 'Family History')
